import fetch from 'isomorphic-fetch';

class Func_Zqmb {

    constructor() {
        this.params = "";
        //MSG保留关键字
        this.msgSequence = null;  //系统流水号（全调用链路唯一，比如调用链路C->S1->S2->S3，这4个调用的sys.sequence保持一致）
        this.msgDebug = Zqmb.debug;  //是否开启debug调试
        this.msgTimeout = Zqmb.timeout; //超时时间设置,服务器默认为30000
        this.msgBypass = Zqmb.bypass;  //N-不备份, Y-备份头 A-备份头+体 ，默认备份
        this.callback = "";
        this.authSysid = Zqmb.sysid; //系统编号
        this.authToken = Zqmb.token; //令牌
        this.authPermit = Zqmb.permit; //许可证
        /*BIZ保留关键字,设置到message头部，也可以直接设置*/
        this.bSequence = null;  //业务流水号
        this.bAccount = Zqmb.account;   //业务账号
        this.headers = { 'msg.callback': this.callback, 'auth.sysid': this.authSysid, 'auth.permit': this.authPermit };
        this.fetch = fetch;
    }

    sequence = msgSequence => {
        this.msgSequence = msgSequence || null;
        return this;
    }
    //设置是否开启debug调试
    debug = msgDebug => {
        this.msgDebug = msgDebug || Zqmb.debug;
        return this;
    }
    //设置超时时间
    timeout = msgTimeout => {
        if (!/^[0-9]*$/.test(msgTimeout)) {
            throw new Error("timeout 格式不正确!");
            return false;
        }
        this.msgTimeout = msgTimeout || Zqmb.timeout;
        return this;
    }
    /*设置是否备份*/
    bypass = msgBypass => {
        this.msgBypass = msgBypass || Zqmb.bypass;
        return this;
    }
    /*设置系统编号*/
    sysid = authSysid => {
        if (!/^[0-9]*$/.test(authSysid)) {
            throw new Error("authSysid 格式不正确!");
            return false;
        }
        this.authSysid = authSysid || Zqmb.sysid;
        return this;
    }
    /*设置令牌*/
    token = authToken => {
        this.authToken = authToken || Zqmb.token;
        return this;
    }
    /*设置许可证*/
    permit = authPermit => {
        this.authPermit = authPermit || Zqmb.permit;
        return this;
    }
    /*设置业务流水号*/
    bizSequence = bSequence => {
        this.bSequence = bSequence || null;
        return this;
    }
    /*设置业务账号*/
    bizAccount = bAccount => {
        this.bAccount = bAccount || Zqmb.account;
        return this;
    }

    initHeaders = () => {
        if (this.msgSequence) {
            this.headers['msg.sequence'] = this.msgSequence;
        }
        if (this.msgDebug) {
            this.headers['msg.debug'] = "Y";
        }
        if (this.msgTimeout) {
            this.headers['msg.timeout'] = this.msgTimeout;
        }
        if (this.msgBypass) {
            this.headers['msg.bypass'] = this.msgBypass;
        }
        if (this.authSysid) {
            this.headers["auth.sysid"] = this.authSysid;
        }
        if (this.authToken) {
            this.headers["auth.token"] = this.authToken;
        }
        if (this.authPermit) {
            this.headers["auth.permit"] = this.authPermit;
        }
        if (this.bSequence) {
            this.headers["biz.sequence"] = this.bSequence;
        }
        if (this.bAccount) {
            this.headers["biz.account"] = this.bAccount;
        }
    }

    get = (url, ...inparams) => {
        if (!url) {
            throw new Error("get方法必须带一个url参数");
            return;
        }
        if (!this.checkUrl(url)) {
            url = Zqmb.host + url;
        }
        this.initHeaders();
        let m_params = inparams.map((param, index) => {
            if (typeof param == "string") {
                param = param.replace(/\\/g,"\\\\");
                param = param.replace(/\"/g, "\\\"");
                return "\"" + param + "\"";
            } else if (typeof param == "object") {
                return "'" + JSON.stringify(param) + "'";
            }
            return param;
        });
        this.params = m_params.join(",");
        let options = { method: 'POST', headers: this.headers };
        if (this.params) {
            options.body = this.params;
        }
        return this.fetch(url, options);
    }

    post = (url, ...inparams) => {
        if (!url) {
            throw new Error("post方法必须带一个url参数");
            return;
        }
        if (!this.checkUrl(url)) {
            url = Zqmb.host + url;
        }
        this.initHeaders();
        let m_params = inparams.map((param, index) => {
            if (typeof param == "string") {
                param = param.replace(/\\/g,"\\\\");
                param = param.replace(/\"/g, "\\\"");
                return "\"" + param + "\"";
            } else if (typeof param == "object") {
                return "'" + JSON.stringify(param) + "'";
            } else if (typeof param == "undefined") {
                return "";
            }
            return param;
        });
        this.params = m_params.join(",");
        let options = { method: 'POST', headers: this.headers };
        if (this.params) {
            options.body = this.params;
        }
        return this.fetch(url, options);
    }

    /*验证url*/
    checkUrl = (url) => {
        var reg = /^(http|ftp|https):\/\/[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&:/~\+#]*[\w\-\@?^=%&/~\+#])?/;
        return reg.test(url);
    }

}

const Zqmb = {
    me: () => {
        return new Func_Zqmb();
    },
    host: "",
    debug: false,
    timeout: null,
    bypass: "",
    sysid: "1002",
    token: "",
    permit: "exhqPb62I2BtMlaj5kkoJEWRFbvfxc",
    account: null,
    login: (authType, userid, passwd, authFactor, ext) => {
        return Zqmb.me().post("/*/buadmin/authenticate", authType, userid, passwd, authFactor, ext)
            .then(res => res.text())
            .catch(err => {
                throw err
            });
    }
}
// commonjs
module.exports = Zqmb;
// es6 default export compatibility
module.exports.default = module.exports;